#pragma once
#include <tokenizers_cpp.h>

#include <memory>

#include "api/base_op.h"

namespace fg {

using absl::optional;
using std::string;
using std::vector;
using tokenizers::Tokenizer;

/**
 * @brief 分词特征：输入字符串，输出是一个数组
 */
class BPETokenizer : public IFeatureOP {
 public:
  int Initialize(const string& feature_config) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithStrOutputs(const vector<Field>& inputs,
                            vector<string>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithInt32Outputs(const vector<Field>& inputs,
                              vector<int32>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithInt64Outputs(const vector<Field>& inputs,
                              vector<int64>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithFloatOutputs(const vector<Field>& inputs,
                              vector<float>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithDoubleOutputs(const vector<Field>& inputs,
                               vector<double>& outputs) override;

 protected:
  const size_t vocab_size() const { return tokenizer_->GetVocabSize(); }

 private:
  vector<int> default_values_;
  string feature_name_;
  string dval_str_;
  string vocab_file_;
  string output_type_;
  std::unique_ptr<Tokenizer> tokenizer_;
};

}  // end of namespace fg

REGISTER_PLUGIN("BPETokenize", BPETokenizer);