#!/usr/bin/env bash
# Scripts to build and install native C++ libraries
set -eu
echo "run build.sh $@"
CURDIR=$(dirname $(realpath $0))
echo "dir: $CURDIR"

[[ -z ${CMAKE:-} ]] && CMAKE="cmake"
[[ -z ${CTEST:-} ]] && CTEST="ctest"
[[ -z ${MAKEJ:-} ]] && MAKEJ=4
[[ -z ${OLDCC:-} ]] && OLDCC="gcc"
[[ -z ${OLDCXX:-} ]] && OLDCXX="g++"
[[ -z ${OLDFC:-} ]] && OLDFC="gfortran"

KERNEL=(`uname -s | tr '[A-Z]' '[a-z]'`)
ARCH=(`uname -m | tr '[A-Z]' '[a-z]'`)
case $KERNEL in
    darwin)
        OS=macosx
        ;;
    mingw32*)
        OS=windows
        KERNEL=windows
        ARCH=x86
        ;;
    mingw64*)
        OS=windows
        KERNEL=windows
        ARCH=x86_64
        ;;
    *)
        OS=$KERNEL
        ;;
esac
case $ARCH in
    arm64)
        ARCH=arm64
        ;;
    arm*)
        ARCH=arm
        ;;
    aarch64*)
        ARCH=arm64
        ;;
    i386|i486|i586|i686)
        ARCH=x86
        ;;
    amd64|x86-64)
        ARCH=x86_64
        ;;
esac
PLATFORM=$OS-$ARCH
echo "Detected platform \"$PLATFORM\""

BUILD_WITH_AVX512=OFF
if [ $# -gt 1 ] && [ "${2:-}" == "avx" ]; then
  BUILD_WITH_AVX512=ON
fi

while [[ $# > 0 ]]; do
    case "$1" in
        build)
            OPERATION=build
            ;;
        clean)
            OPERATION=clean
            ;;
        test)
            OPERATION=test
            ;;
        debug)
            OPERATION=debug
            ;;
        *)
            PROJECTS+=("$1")
            ;;
    esac
    shift
done

BUILD_TESTS=OFF
BUILD_TYPE=Release
OP_BLD_DIR=${CURDIR}/build

function build() {
  export TMPDIR=${OP_BLD_DIR}/tmp
  mkdir -p ${OP_BLD_DIR}/tmp
  source /opt/rh/devtoolset-11/enable
  ${CMAKE} --version
  ${CMAKE} --no-warn-unused-cli \
  -DCMAKE_TOOLCHAIN_FILE=/opt/vcpkg/scripts/buildsystems/vcpkg.cmake \
  -DCMAKE_EXPORT_COMPILE_COMMANDS:BOOL=TRUE \
  -DCMAKE_BUILD_TYPE:STRING=${BUILD_TYPE} \
  -DBUILD_TESTS=${BUILD_TESTS} \
  -DCMAKE_CXX_FLAGS="-Wp,-I/tmpdir=${OP_BLD_DIR}/tmp" \
  -S ${CURDIR} -B ${OP_BLD_DIR}

  ${CMAKE} --build ${OP_BLD_DIR} --config Release --target all -- -j48
  ${CMAKE} --build ${OP_BLD_DIR} --config Release --target package
}


if [ "${OPERATION:-build}" == "build" ]; then
  build
elif [ "${OPERATION:-build}" == "debug" ]; then
  BUILD_TESTS=ON
  BUILD_TYPE=Debug
  build
elif [ "${OPERATION:-build}" == "clean" ]; then
  cd build
  make clean
  cd -
elif [ "${OPERATION:-build}" == "test" ]; then
  cd build
  ${CTEST} -j 24
  cd -
fi
